/*
 * Copyright (C) 2020 Sebastiano Vigna
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package it.unimi.dsi.fastutil.ints;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.Collections;
import java.util.HashMap;

import org.junit.Test;

public class Int2IntArrayMapTest {
	@Test
	public void testCopyConstructor() {
		final Int2IntOpenHashMap m = new Int2IntOpenHashMap(new int [] {1, 2}, new int[] {3, 4});
		assertEquals(new Int2IntArrayMap(m), m);
		assertEquals(new HashMap<>(m), m);
	}

	@Test
	public void testValuesClear() {
		final Int2IntMap map = new Int2IntArrayMap(Int2IntMaps.singleton(42, 24));
		map.values().clear();
		assertTrue(map.isEmpty());
	}

	@Test
	public void testValuesRemove() {
		final Int2IntMap map = new Int2IntArrayMap(Int2IntMaps.singleton(42, 24));
		map.values().rem(24);
		assertTrue(map.isEmpty());
	}

	@Test
	public void testValuesRemoveAll() {
		final Int2IntMap map = new Int2IntArrayMap(Int2IntMaps.singleton(42, 24));
		map.values().removeAll(Collections.singleton(Integer.valueOf(24)));
		assertTrue(map.isEmpty());
	}
}
